@@ -35,6 +35,9 @@ import (
 	"k8s.io/apimachinery/pkg/runtime"
 	"k8s.io/apimachinery/pkg/util/sets"
 	"k8s.io/klog/v2"
+	"k8s.io/kubectl/pkg/util/i18n"
+	"k8s.io/kubectl/pkg/util/templates"
+
 	kopsbase "k8s.io/kops"
 	"k8s.io/kops/cmd/kops/util"
 	api "k8s.io/kops/pkg/apis/kops"
@@ -48,13 +51,11 @@ import (
 	"k8s.io/kops/pkg/commands/commandutils"
 	"k8s.io/kops/pkg/featureflag"
 	"k8s.io/kops/pkg/kubeconfig"
-	"k8s.io/kops/pkg/kubemanifest"
+	"k8s.io/kops/pkg/wellknownoperators"
 	"k8s.io/kops/pkg/zones"
 	"k8s.io/kops/upup/pkg/fi"
 	"k8s.io/kops/upup/pkg/fi/cloudup"
 	"k8s.io/kops/upup/pkg/fi/utils"
-	"k8s.io/kubectl/pkg/util/i18n"
-	"k8s.io/kubectl/pkg/util/templates"
 )
 
 type CreateClusterOptions struct {
@@ -314,9 +315,7 @@ func NewCmdCreateCluster(f *util.Factory, out io.Writer) *cobra.Command {
 	// TODO: Can we deprecate this flag - it is awkward?
 	cmd.Flags().BoolVar(&associatePublicIP, "associate-public-ip", false, "Specify --associate-public-ip=[true|false] to enable/disable association of public IP for master ASG and nodes. Default is 'true'.")
 
-	if featureflag.AWSIPv6.Enabled() {
-		cmd.Flags().BoolVar(&options.IPv6, "ipv6", false, "Allocate IPv6 CIDRs to subnets for clusters with public topology (AWS only)")
-	}
+	cmd.Flags().BoolVar(&options.IPv6, "ipv6", false, "Use IPv6 for the pod network (AWS only)")
 
 	cmd.Flags().StringSliceVar(&options.NodeSecurityGroups, "node-security-groups", options.NodeSecurityGroups, "Additional precreated security groups to add to worker nodes.")
 	cmd.RegisterFlagCompletionFunc("node-security-groups", completeSecurityGroup)
@@ -449,6 +448,12 @@ func NewCmdCreateCluster(f *util.Factory, out io.Writer) *cobra.Command {
 		return pflag.NormalizedName(name)
 	})
 
+	if featureflag.Karpenter.Enabled() {
+		cmd.Flags().StringVar(&options.InstanceManager, "instance-manager", options.InstanceManager, "Instance manager to use (cloudgroups or karpenter. Default: cloudgroups)")
+		cmd.RegisterFlagCompletionFunc("instance-manager", func(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
+			return []string{"cloudgroups", "karpenter"}, cobra.ShellCompDirectiveNoFileComp
+		})
+	}
 	return cmd
 }
 
@@ -664,12 +669,30 @@ func RunCreateCluster(ctx context.Context, f *util.Factory, out io.Writer, c *Cr
 			return err
 		}
 		fullGroup.AddInstanceGroupNodeLabel()
-		if api.CloudProviderID(cluster.Spec.CloudProvider) == api.CloudProviderGCE {
+		if cluster.Spec.GetCloudProvider() == api.CloudProviderGCE {
 			fullGroup.Spec.NodeLabels["cloud.google.com/metadata-proxy-ready"] = "true"
 		}
 		fullInstanceGroups = append(fullInstanceGroups, fullGroup)
 	}
 
+	kubernetesVersion, err := kopsutil.ParseKubernetesVersion(clusterResult.Cluster.Spec.KubernetesVersion)
+	if err != nil {
+		return fmt.Errorf("cannot parse KubernetesVersion %q in cluster: %w", clusterResult.Cluster.Spec.KubernetesVersion, err)
+	}
+
+	addons, err := wellknownoperators.CreateAddons(clusterResult.Channel, kubernetesVersion, fullCluster)
+	if err != nil {
+		return err
+	}
+
+	for _, p := range c.AddonPaths {
+		addon, err := clusteraddons.LoadClusterAddon(p)
+		if err != nil {
+			return fmt.Errorf("error loading cluster addon %s: %v", p, err)
+		}
+		addons = append(addons, addon.Objects...)
+	}
+
 	err = validation.DeepValidate(fullCluster, fullInstanceGroups, true, nil)
 	if err != nil {
 		return err
@@ -685,6 +708,11 @@ func RunCreateCluster(ctx context.Context, f *util.Factory, out io.Writer, c *Cr
 			group.ObjectMeta.Labels[api.LabelClusterName] = cluster.ObjectMeta.Name
 			obj = append(obj, group)
 		}
+
+		for _, o := range addons {
+			obj = append(obj, o.ToUnstructured())
+		}
+
 		switch c.Output {
 		case OutputYaml:
 			if err := fullOutputYAML(out, obj...); err != nil {
@@ -701,15 +729,6 @@ func RunCreateCluster(ctx context.Context, f *util.Factory, out io.Writer, c *Cr
 		}
 	}
 
-	var addons kubemanifest.ObjectList
-	for _, p := range c.AddonPaths {
-		addon, err := clusteraddons.LoadClusterAddon(p)
-		if err != nil {
-			return fmt.Errorf("error loading cluster addon %s: %v", p, err)
-		}
-		addons = append(addons, addon.Objects...)
-	}
-
 	// Note we perform as much validation as we can, before writing a bad config
 	err = registry.CreateClusterConfig(ctx, clientset, cluster, fullInstanceGroups, addons)
 	if err != nil {
@@ -880,7 +899,7 @@ func completeKubernetesVersion(cmd *cobra.Command, args []string, toComplete str
 	tooNewVersion.Pre = nil
 	tooNewVersion.Build = nil
 
-	repo, err := name.NewRepository("k8s.gcr.io/kube-apiserver")
+	repo, err := name.NewRepository("registry.k8s.io/kube-apiserver")
 	if err != nil {
 		return commandutils.CompletionError("parsing kube-apiserver repo", err)
 	}
@@ -966,7 +985,7 @@ func completeNetworking(options *CreateClusterOptions) func(cmd *cobra.Command,
 				completions = append(completions, "amazonvpc")
 			}
 
-			if featureflag.AlphaAllowGCE.Enabled() && (options.CloudProvider == "gce" || options.CloudProvider == "") {
+			if options.CloudProvider == "gce" || options.CloudProvider == "" {
 				completions = append(completions, "gce")
 			}
 		}
