@@ -23,22 +23,18 @@ import (
 	"os"
 	"path"
 	"path/filepath"
-
-	"k8s.io/kops/pkg/model/components"
+	"strings"
 
 	"github.com/aws/aws-sdk-go/aws/ec2metadata"
 	"github.com/aws/aws-sdk-go/aws/session"
 	"github.com/aws/aws-sdk-go/service/ec2"
-
 	v1 "k8s.io/api/core/v1"
-	"k8s.io/klog/v2"
-	kubelet "k8s.io/kubelet/config/v1beta1"
-
 	"k8s.io/apimachinery/pkg/runtime"
 	"k8s.io/apimachinery/pkg/runtime/serializer"
-
+	"k8s.io/klog/v2"
 	"k8s.io/kops/pkg/apis/kops"
 	"k8s.io/kops/pkg/flagbuilder"
+	"k8s.io/kops/pkg/model/components"
 	"k8s.io/kops/pkg/nodelabels"
 	"k8s.io/kops/pkg/rbac"
 	"k8s.io/kops/pkg/systemd"
@@ -46,6 +42,7 @@ import (
 	"k8s.io/kops/upup/pkg/fi/cloudup/awsup"
 	"k8s.io/kops/upup/pkg/fi/nodeup/nodetasks"
 	"k8s.io/kops/util/pkg/distributions"
+	kubelet "k8s.io/kubelet/config/v1beta1"
 )
 
 const (
@@ -137,14 +134,7 @@ func (b *KubeletBuilder) Build(c *fi.ModelBuilderContext) error {
 		if b.HasAPIServer || !b.UseBootstrapTokens() {
 			var kubeconfig fi.Resource
 			if b.HasAPIServer {
-				if b.IsKubernetesGTE("1.19") || b.UseBootstrapTokens() {
-					kubeconfig, err = b.buildMasterKubeletKubeconfig(c)
-				} else {
-					kubeconfig = b.BuildIssuedKubeconfig("kubelet", nodetasks.PKIXName{
-						CommonName:   "kubelet",
-						Organization: []string{rbac.NodesGroup},
-					}, c)
-				}
+				kubeconfig, err = b.buildMasterKubeletKubeconfig(c)
 			} else {
 				kubeconfig, err = b.BuildBootstrapKubeconfig("kubelet", c)
 			}
@@ -173,6 +163,49 @@ func (b *KubeletBuilder) Build(c *fi.ModelBuilderContext) error {
 		return err
 	}
 
+	if kubeletConfig.CgroupDriver == "systemd" && b.Cluster.Spec.ContainerRuntime == "containerd" {
+
+		{
+			cgroup := kubeletConfig.KubeletCgroups
+			if cgroup != "" {
+				c.EnsureTask(b.buildCgroupService(cgroup))
+			}
+
+		}
+		{
+			cgroup := kubeletConfig.RuntimeCgroups
+			if cgroup != "" {
+				c.EnsureTask(b.buildCgroupService(cgroup))
+			}
+
+		}
+		/* Kubelet incorrectly interprets this value when CgroupDriver is systemd
+		See https://github.com/kubernetes/kubernetes/issues/101189
+		{
+			cgroup := kubeletConfig.KubeReservedCgroup
+			if cgroup != "" {
+				c.EnsureTask(b.buildCgroupService(cgroup))
+			}
+		}
+		*/
+
+		{
+			cgroup := kubeletConfig.SystemCgroups
+			if cgroup != "" {
+				c.EnsureTask(b.buildCgroupService(cgroup))
+			}
+		}
+
+		/* This suffers from the same issue as KubeReservedCgroup
+		{
+			cgroup := kubeletConfig.SystemReservedCgroup
+			if cgroup != "" {
+				c.EnsureTask(b.buildCgroupService(cgroup))
+			}
+		}
+		*/
+	}
+
 	c.AddTask(b.buildSystemdService())
 
 	return nil
@@ -258,13 +291,13 @@ func (b *KubeletBuilder) buildSystemdEnvironmentFile(kubeletConfig *kops.Kubelet
 	}
 
 	if b.UsesSecondaryIP() {
-		sess := session.Must(session.NewSession())
-		metadata := ec2metadata.New(sess)
-		localIpv4, err := metadata.GetMetadata("local-ipv4")
+		localIP, err := b.GetMetadataLocalIP()
 		if err != nil {
-			return nil, fmt.Errorf("error fetching the local-ipv4 address from the ec2 meta-data: %v", err)
+			return nil, err
+		}
+		if localIP != "" {
+			flags += " --node-ip=" + localIP
 		}
-		flags += " --node-ip=" + localIpv4
 	}
 
 	if b.usesContainerizedMounter() {
@@ -274,11 +307,16 @@ func (b *KubeletBuilder) buildSystemdEnvironmentFile(kubeletConfig *kops.Kubelet
 
 	// Add container runtime spcific flags
 	switch b.Cluster.Spec.ContainerRuntime {
-	case "docker", "":
-		flags += " --cni-bin-dir=" + b.CNIBinDir()
-		flags += " --cni-conf-dir=" + b.CNIConfDir()
+	case "docker":
+		if b.IsKubernetesLT("1.24") {
+			flags += " --container-runtime=docker"
+			flags += " --cni-bin-dir=" + b.CNIBinDir()
+			flags += " --cni-conf-dir=" + b.CNIConfDir()
+		}
 	case "containerd":
-		flags += " --container-runtime=remote"
+		if b.IsKubernetesLT("1.24") {
+			flags += " --container-runtime=remote"
+		}
 		flags += " --runtime-request-timeout=15m"
 		if b.Cluster.Spec.Containerd == nil || b.Cluster.Spec.Containerd.Address == nil {
 			flags += " --container-runtime-endpoint=unix:///run/containerd/containerd.sock"
@@ -346,6 +384,13 @@ func (b *KubeletBuilder) buildSystemdService() *nodetasks.Service {
 
 	manifest.Set("Install", "WantedBy", "multi-user.target")
 
+	if b.Cluster.Spec.Kubelet.CgroupDriver == "systemd" && b.Cluster.Spec.ContainerRuntime == "containerd" {
+		cgroup := b.Cluster.Spec.Kubelet.KubeletCgroups
+		if cgroup != "" {
+			manifest.Set("Service", "Slice", strings.Trim(cgroup, "/")+".slice")
+		}
+	}
+
 	manifestString := manifest.Render()
 
 	klog.V(8).Infof("Built service manifest %q\n%s", "kubelet", manifestString)
@@ -539,7 +584,11 @@ func (b *KubeletBuilder) buildKubeletConfigSpec() (*kops.KubeletConfigSpec, erro
 	{
 		if len(c.Taints) == 0 && isMaster {
 			// (Even though the value is empty, we still expect <Key>=<Value>:<Effect>)
-			c.Taints = append(c.Taints, nodelabels.RoleLabelMaster16+"=:"+string(v1.TaintEffectNoSchedule))
+			if b.IsKubernetesLT("1.24") {
+				c.Taints = append(c.Taints, nodelabels.RoleLabelMaster16+"=:"+string(v1.TaintEffectNoSchedule))
+			} else {
+				c.Taints = append(c.Taints, nodelabels.RoleLabelControlPlane20+"=:"+string(v1.TaintEffectNoSchedule))
+			}
 		}
 		if len(c.Taints) == 0 && isAPIServer {
 			// (Even though the value is empty, we still expect <Key>=<Value>:<Effect>)
@@ -578,11 +627,11 @@ func (b *KubeletBuilder) buildKubeletConfigSpec() (*kops.KubeletConfigSpec, erro
 	// For bootstrapping reasons, protokube sets the critical labels for kops-controller to run.
 	c.NodeLabels = nil
 
-	if c.AuthorizationMode == "" && b.Cluster.IsKubernetesGTE("1.19") {
+	if c.AuthorizationMode == "" {
 		c.AuthorizationMode = "Webhook"
 	}
 
-	if c.AuthenticationTokenWebhook == nil && b.Cluster.IsKubernetesGTE("1.19") {
+	if c.AuthenticationTokenWebhook == nil {
 		c.AuthenticationTokenWebhook = fi.Bool(true)
 	}
 
@@ -654,7 +703,7 @@ func (b *KubeletBuilder) buildKubeletServingCertificate(c *fi.ModelBuilderContex
 }
 
 func (b *KubeletBuilder) kubeletNames() ([]string, error) {
-	if kops.CloudProviderID(b.Cluster.Spec.CloudProvider) != kops.CloudProviderAWS {
+	if b.CloudProvider != kops.CloudProviderAWS {
 		name, err := os.Hostname()
 		if err != nil {
 			return nil, err
@@ -674,6 +723,24 @@ func (b *KubeletBuilder) kubeletNames() ([]string, error) {
 		return nil, fmt.Errorf("error describing instances: %v", err)
 	}
 
-	useInstanceIDForNodeName := b.Cluster.Spec.ExternalCloudControllerManager != nil && b.IsKubernetesGTE("1.22")
+	useInstanceIDForNodeName := b.Cluster.Spec.ExternalCloudControllerManager != nil && b.IsKubernetesGTE("1.23")
 	return awsup.GetInstanceCertificateNames(result, useInstanceIDForNodeName)
 }
+
+func (b *KubeletBuilder) buildCgroupService(name string) *nodetasks.Service {
+	name = strings.Trim(name, "/")
+
+	manifest := &systemd.Manifest{}
+	manifest.Set("Unit", "Documentation", "man:systemd.special(7)")
+	manifest.Set("Unit", "Before", "slices.target")
+	manifest.Set("Unit", "DefaultDependencies", "no")
+
+	manifestString := manifest.Render()
+
+	service := &nodetasks.Service{
+		Name:       name + ".slice",
+		Definition: s(manifestString),
+	}
+
+	return service
+}
