@@ -185,7 +185,9 @@ func TestValidMasterInstanceGroup(t *testing.T) {
 func TestValidBootDevice(t *testing.T) {
 	cluster := &kops.Cluster{
 		Spec: kops.ClusterSpec{
-			CloudProvider: "aws",
+			CloudProvider: kops.CloudProviderSpec{
+				AWS: &kops.AWSSpec{},
+			},
 		},
 	}
 	grid := []struct {
@@ -218,16 +220,9 @@ func TestValidBootDevice(t *testing.T) {
 	}
 
 	for _, g := range grid {
-		ig := &kops.InstanceGroup{
-			ObjectMeta: v1.ObjectMeta{
-				Name: "some-ig",
-			},
-			Spec: kops.InstanceGroupSpec{
-				Role:           "Node",
-				RootVolumeType: fi.String(g.volumeType),
-			},
-		}
-		errs := CrossValidateInstanceGroup(ig, cluster, nil)
+		ig := createMinimalInstanceGroup()
+		ig.Spec.RootVolumeType = fi.String(g.volumeType)
+		errs := CrossValidateInstanceGroup(ig, cluster, nil, true)
 		testErrors(t, g.volumeType, errs, g.expected)
 	}
 }
@@ -253,17 +248,11 @@ func TestValidNodeLabels(t *testing.T) {
 	}
 
 	for _, g := range grid {
-		ig := &kops.InstanceGroup{
-			ObjectMeta: v1.ObjectMeta{
-				Name: "some-ig",
-			},
-			Spec: kops.InstanceGroupSpec{
-				Role:       "Node",
-				NodeLabels: make(map[string]string),
-			},
-		}
+
+		ig := createMinimalInstanceGroup()
+		ig.Spec.NodeLabels = make(map[string]string)
 		ig.Spec.NodeLabels[g.label] = "placeholder"
-		errs := ValidateInstanceGroup(ig, nil)
+		errs := ValidateInstanceGroup(ig, nil, true)
 		testErrors(t, g.label, errs, g.expected)
 	}
 }
@@ -289,17 +278,10 @@ func TestValidateIGCloudLabels(t *testing.T) {
 	}
 
 	for _, g := range grid {
-		ig := &kops.InstanceGroup{
-			ObjectMeta: v1.ObjectMeta{
-				Name: "some-ig",
-			},
-			Spec: kops.InstanceGroupSpec{
-				Role:        "Node",
-				CloudLabels: make(map[string]string),
-			},
-		}
+		ig := createMinimalInstanceGroup()
+
 		ig.Spec.CloudLabels[g.label] = "placeholder"
-		errs := ValidateInstanceGroup(ig, nil)
+		errs := ValidateInstanceGroup(ig, nil, true)
 		testErrors(t, g.label, errs, g.expected)
 	}
 }
@@ -319,17 +301,10 @@ func TestIGCloudLabelIsIGName(t *testing.T) {
 	}
 
 	for _, g := range grid {
-		ig := &kops.InstanceGroup{
-			ObjectMeta: v1.ObjectMeta{
-				Name: "some-ig",
-			},
-			Spec: kops.InstanceGroupSpec{
-				Role:        "Node",
-				CloudLabels: make(map[string]string),
-			},
-		}
+		ig := createMinimalInstanceGroup()
+
 		ig.Spec.CloudLabels[aws.CloudTagInstanceGroupName] = g.label
-		errs := ValidateInstanceGroup(ig, nil)
+		errs := ValidateInstanceGroup(ig, nil, true)
 		testErrors(t, g.label, errs, g.expected)
 	}
 }
@@ -363,7 +338,7 @@ func TestValidTaints(t *testing.T) {
 		ig := createMinimalInstanceGroup()
 
 		ig.Spec.Taints = g.taints
-		errs := ValidateInstanceGroup(ig, nil)
+		errs := ValidateInstanceGroup(ig, nil, true)
 		testErrors(t, g.taints, errs, g.expected)
 	}
 }
@@ -397,18 +372,11 @@ func TestIGUpdatePolicy(t *testing.T) {
 			expected: []string{unsupportedValueError},
 		},
 	} {
-		ig := kops.InstanceGroup{
-			ObjectMeta: v1.ObjectMeta{
-				Name: "some-ig",
-			},
-			Spec: kops.InstanceGroupSpec{
-				Role:        "Node",
-				CloudLabels: make(map[string]string),
-			},
-		}
+		ig := createMinimalInstanceGroup()
+
 		t.Run(test.label, func(t *testing.T) {
 			ig.Spec.UpdatePolicy = test.policy
-			errs := ValidateInstanceGroup(&ig, nil)
+			errs := ValidateInstanceGroup(ig, nil, true)
 			testErrors(t, test.label, errs, test.expected)
 		})
 	}
@@ -428,6 +396,9 @@ func TestValidInstanceGroup(t *testing.T) {
 				Spec: kops.InstanceGroupSpec{
 					Role:    kops.InstanceGroupRoleMaster,
 					Subnets: []string{"eu-central-1a"},
+					MaxSize: fi.Int32(1),
+					MinSize: fi.Int32(1),
+					Image:   "my-image",
 				},
 			},
 			ExpectedErrors: 0,
@@ -441,6 +412,9 @@ func TestValidInstanceGroup(t *testing.T) {
 				Spec: kops.InstanceGroupSpec{
 					Role:    kops.InstanceGroupRoleAPIServer,
 					Subnets: []string{"eu-central-1a"},
+					MaxSize: fi.Int32(1),
+					MinSize: fi.Int32(1),
+					Image:   "my-image",
 				},
 			},
 			ExpectedErrors: 0,
@@ -454,6 +428,9 @@ func TestValidInstanceGroup(t *testing.T) {
 				Spec: kops.InstanceGroupSpec{
 					Role:    kops.InstanceGroupRoleNode,
 					Subnets: []string{"eu-central-1a"},
+					MaxSize: fi.Int32(1),
+					MinSize: fi.Int32(1),
+					Image:   "my-image",
 				},
 			},
 			ExpectedErrors: 0,
@@ -467,6 +444,9 @@ func TestValidInstanceGroup(t *testing.T) {
 				Spec: kops.InstanceGroupSpec{
 					Role:    kops.InstanceGroupRoleBastion,
 					Subnets: []string{"eu-central-1a"},
+					MaxSize: fi.Int32(1),
+					MinSize: fi.Int32(1),
+					Image:   "my-image",
 				},
 			},
 			ExpectedErrors: 0,
@@ -474,7 +454,7 @@ func TestValidInstanceGroup(t *testing.T) {
 		},
 	}
 	for _, g := range grid {
-		errList := ValidateInstanceGroup(g.IG, nil)
+		errList := ValidateInstanceGroup(g.IG, nil, true)
 		testErrors(t, g.Description, errList, []string{})
 	}
 }
