@@ -21,11 +21,12 @@ import (
 	"strings"
 
 	"k8s.io/klog/v2"
+	"k8s.io/legacy-cloud-providers/aws"
+
 	"k8s.io/kops/pkg/apis/kops"
 	"k8s.io/kops/upup/pkg/fi"
 	"k8s.io/kops/upup/pkg/fi/cloudup/awstasks"
 	"k8s.io/kops/upup/pkg/fi/cloudup/awsup"
-	"k8s.io/legacy-cloud-providers/aws"
 )
 
 // NetworkModelBuilder configures network objects
@@ -157,7 +158,7 @@ func (b *NetworkModelBuilder) Build(c *fi.ModelBuilderContext) error {
 
 		if !isUnmanaged(subnetSpec) {
 			allSubnetsUnmanaged = false
-			if subnetSpec.Type == kops.SubnetTypePrivate {
+			if subnetSpec.Type == kops.SubnetTypeDualStack || subnetSpec.Type == kops.SubnetTypePrivate {
 				allPrivateSubnetsUnmanaged = false
 			}
 		}
@@ -215,6 +216,13 @@ func (b *NetworkModelBuilder) Build(c *fi.ModelBuilderContext) error {
 
 	infoByZone := make(map[string]*zoneInfo)
 
+	haveDualStack := map[string]bool{}
+	for _, subnetSpec := range b.Cluster.Spec.Subnets {
+		if subnetSpec.Type == kops.SubnetTypeDualStack {
+			haveDualStack[subnetSpec.Zone] = true
+		}
+	}
+
 	for i := range b.Cluster.Spec.Subnets {
 		subnetSpec := &b.Cluster.Spec.Subnets[i]
 		sharedSubnet := subnetSpec.ProviderID != ""
@@ -237,9 +245,14 @@ func (b *NetworkModelBuilder) Build(c *fi.ModelBuilderContext) error {
 					tags[aws.TagNameSubnetInternalELB] = "1"
 				}
 
-			case kops.SubnetTypePrivate:
+			case kops.SubnetTypeDualStack:
 				tags[aws.TagNameSubnetInternalELB] = "1"
 
+			case kops.SubnetTypePrivate:
+				if !haveDualStack[subnetSpec.Zone] {
+					tags[aws.TagNameSubnetInternalELB] = "1"
+				}
+
 			default:
 				klog.V(2).Infof("unable to properly tag subnet %q because it has unknown type %q. Load balancers may be created in incorrect subnets", subnetSpec.Name, subnetSpec.Type)
 			}
@@ -253,11 +266,18 @@ func (b *NetworkModelBuilder) Build(c *fi.ModelBuilderContext) error {
 			Lifecycle:        b.Lifecycle,
 			VPC:              b.LinkToVPC(),
 			AvailabilityZone: fi.String(subnetSpec.Zone),
-			CIDR:             fi.String(subnetSpec.CIDR),
 			Shared:           fi.Bool(sharedSubnet),
 			Tags:             tags,
 		}
 
+		if b.Cluster.Spec.ExternalCloudControllerManager != nil && b.Cluster.IsKubernetesGTE("1.22") {
+			subnet.ResourceBasedNaming = fi.Bool(true)
+		}
+
+		if subnetSpec.CIDR != "" {
+			subnet.CIDR = fi.String(subnetSpec.CIDR)
+		}
+
 		if subnetSpec.IPv6CIDR != "" {
 			if !sharedVPC {
 				subnet.AmazonIPv6CIDR = b.LinkToAmazonVPCIPv6CIDR()
@@ -296,7 +316,7 @@ func (b *NetworkModelBuilder) Build(c *fi.ModelBuilderContext) error {
 				}
 			}
 
-		case kops.SubnetTypePrivate:
+		case kops.SubnetTypeDualStack, kops.SubnetTypePrivate:
 			// Private subnets get a Network Gateway, and their own route table to associate them with the network gateway
 
 			if !sharedSubnet && !isUnmanaged(subnetSpec) {
@@ -544,6 +564,22 @@ func (b *NetworkModelBuilder) Build(c *fi.ModelBuilderContext) error {
 					EgressOnlyInternetGateway: eigw,
 				})
 			}
+
+			subnets, err := b.LinkToPrivateSubnetsInZone(zone)
+			if err != nil {
+				return err
+			}
+
+			for _, subnetSpec := range b.Cluster.Spec.Subnets {
+				for _, subnet := range subnets {
+					if strings.HasPrefix(*subnet.Name, subnetSpec.Name) {
+						err := addAdditionalRoutes(subnetSpec.AdditionalRoutes, subnetSpec.Name, rt, b.Lifecycle, c)
+						if err != nil {
+							return err
+						}
+					}
+				}
+			}
 		}
 
 		if info.HaveIPv6PublicSubnet {
@@ -595,3 +631,74 @@ func (b *NetworkModelBuilder) Build(c *fi.ModelBuilderContext) error {
 
 	return nil
 }
+
+func addAdditionalRoutes(routes []kops.RouteSpec, sbName string, rt *awstasks.RouteTable, lf fi.Lifecycle, c *fi.ModelBuilderContext) error {
+	for _, r := range routes {
+		t := &awstasks.Route{
+			Name:       fi.String(sbName + "." + r.CIDR),
+			Lifecycle:  lf,
+			CIDR:       fi.String(r.CIDR),
+			RouteTable: rt,
+		}
+		if strings.HasPrefix(r.Target, "pcx-") {
+			t.VPCPeeringConnectionID = fi.String(r.Target)
+			c.AddTask(t)
+		} else if strings.HasPrefix(r.Target, "i-") {
+			inst := &awstasks.Instance{
+				Name:      fi.String(r.Target),
+				Lifecycle: lf,
+				ID:        fi.String(r.Target),
+				Shared:    fi.Bool(true),
+			}
+			err := c.EnsureTask(inst)
+			if err != nil {
+				return err
+			}
+			t.Instance = inst
+			c.AddTask(t)
+		} else if strings.HasPrefix(r.Target, "nat-") {
+			nat := &awstasks.NatGateway{
+				Name:      fi.String(r.Target),
+				Lifecycle: lf,
+				ID:        fi.String(r.Target),
+				Shared:    fi.Bool(true),
+			}
+			err := c.EnsureTask(nat)
+			if err != nil {
+				return err
+			}
+			t.NatGateway = nat
+			c.AddTask(t)
+		} else if strings.HasPrefix(r.Target, "tgw-") {
+			t.TransitGatewayID = fi.String(r.Target)
+			c.AddTask(t)
+		} else if strings.HasPrefix(r.Target, "igw-") {
+			internetGW := &awstasks.InternetGateway{
+				Name:      fi.String(r.Target),
+				Lifecycle: lf,
+				ID:        fi.String(r.Target),
+				Shared:    fi.Bool(true),
+			}
+			err := c.EnsureTask(internetGW)
+			if err != nil {
+				return err
+			}
+			t.InternetGateway = internetGW
+			c.AddTask(t)
+		} else if strings.HasPrefix(r.Target, "eigw-") {
+			eigw := &awstasks.EgressOnlyInternetGateway{
+				Name:      fi.String(r.Target),
+				Lifecycle: lf,
+				ID:        fi.String(r.Target),
+				Shared:    fi.Bool(true),
+			}
+			err := c.EnsureTask(eigw)
+			if err != nil {
+				return err
+			}
+			t.EgressOnlyInternetGateway = eigw
+			c.AddTask(t)
+		}
+	}
+	return nil
+}
