@@ -66,9 +66,10 @@ func (d *deployer) verifyBuildFlags() error {
 		}
 		d.StageLocation = stageLocation
 	}
-	if d.KopsBaseURL == "" {
+	if d.KopsBaseURL == "" && os.Getenv("KOPS_BASE_URL") == "" {
 		d.KopsBaseURL = strings.Replace(d.StageLocation, "gs://", "https://storage.googleapis.com/", 1)
 	}
+
 	fi, err := os.Stat(d.KopsRoot)
 	if err != nil {
 		return err
@@ -91,7 +92,7 @@ func defaultStageLocation(kopsRoot string) (string, error) {
 		jobName = defaultJobName
 	}
 
-	cmd := exec.Command("git", "describe", "--always")
+	cmd := exec.Command("git", "rev-parse", "--short", "HEAD")
 	cmd.SetDir(kopsRoot)
 	output, err := exec.CombinedOutputLines(cmd)
 	if err != nil {
