@@ -17,7 +17,6 @@ limitations under the License.
 package deployer
 
 import (
-	"crypto/md5"
 	"errors"
 	"fmt"
 	"os"
@@ -30,6 +29,7 @@ import (
 	"k8s.io/kops/tests/e2e/kubetest2-kops/gce"
 	"k8s.io/kops/tests/e2e/pkg/kops"
 	"k8s.io/kops/tests/e2e/pkg/target"
+	"k8s.io/kops/tests/e2e/pkg/util"
 	"sigs.k8s.io/kubetest2/pkg/boskos"
 )
 
@@ -67,13 +67,13 @@ func (d *deployer) initialize() error {
 
 	switch d.CloudProvider {
 	case "aws":
-		// These environment variables are defined by the "preset-aws-ssh" prow preset
-		// https://github.com/kubernetes/test-infra/blob/3d3b325c98b739b526ba5d93ce21c90a05e1f46d/config/prow/config.yaml#L653-L670
-		if d.SSHPrivateKeyPath == "" {
-			d.SSHPrivateKeyPath = os.Getenv("AWS_SSH_PRIVATE_KEY_FILE")
-		}
-		if d.SSHPublicKeyPath == "" {
-			d.SSHPublicKeyPath = os.Getenv("AWS_SSH_PUBLIC_KEY_FILE")
+		if d.SSHPrivateKeyPath == "" || d.SSHPublicKeyPath == "" {
+			publicKeyPath, privateKeyPath, err := util.CreateSSHKeyPair(d.ClusterName)
+			if err != nil {
+				return err
+			}
+			d.SSHPublicKeyPath = publicKeyPath
+			d.SSHPrivateKeyPath = privateKeyPath
 		}
 	case "digitalocean":
 		if d.SSHPrivateKeyPath == "" {
@@ -97,6 +97,7 @@ func (d *deployer) initialize() error {
 				d.boskos,
 				"gce-project",
 				5*time.Minute,
+				5*time.Minute,
 				d.boskosHeartbeatClose,
 			)
 			if err != nil {
@@ -156,19 +157,6 @@ func (d *deployer) verifyKopsFlags() error {
 		return errors.New("missing required --kops-binary-path when --kops-version-marker is not used")
 	}
 
-	// when we use a binary path, we want to use the corresponding nodeup et.al
-	if d.KopsBinaryPath != "" {
-		baseUrl := os.Getenv("KOPS_BASE_URL")
-		if baseUrl == "" {
-			stageLocation, err := defaultStageLocation(d.KopsRoot)
-			if err != nil {
-				return err
-			}
-			d.KopsBaseURL = strings.Replace(stageLocation, "gs://", "https://storage.googleapis.com/", 1)
-		}
-
-	}
-
 	switch d.CloudProvider {
 	case "aws":
 	case "gce":
@@ -198,6 +186,9 @@ func (d *deployer) env() []string {
 				vars = append(vars, k+"="+v)
 			}
 		}
+		// Recognized by the e2e framework
+		// https://github.com/kubernetes/kubernetes/blob/a750d8054a6cb3167f495829ce3e77ab0ccca48e/test/e2e/framework/ssh/ssh.go#L59-L62
+		vars = append(vars, fmt.Sprintf("KUBE_SSH_KEY_PATH=%v", d.SSHPrivateKeyPath))
 	} else if d.CloudProvider == "digitalocean" {
 		// Pass through some env vars if set
 		for _, k := range []string{"DIGITALOCEAN_ACCESS_TOKEN", "S3_ACCESS_KEY_ID", "S3_SECRET_ACCESS_KEY"} {
@@ -221,7 +212,6 @@ func (d *deployer) env() []string {
 func (d *deployer) featureFlags() string {
 	ff := []string{
 		"+SpecOverrideFlag",
-		"+AlphaAllowGCE",
 	}
 	val := strings.Join(ff, ",")
 	for _, env := range d.Env {
@@ -246,9 +236,6 @@ func defaultClusterName(cloudProvider string) (string, error) {
 		return "", errors.New("PULL_NUMBER must be set when JOB_TYPE=presubmit and --cluster-name is not set")
 	}
 
-	buildIDHash := fmt.Sprintf("%x", md5.Sum([]byte(buildID)))
-	jobHash := fmt.Sprintf("%x", md5.Sum([]byte(jobName)))
-
 	var suffix string
 	switch cloudProvider {
 	case "aws":
@@ -258,10 +245,9 @@ func defaultClusterName(cloudProvider string) (string, error) {
 	}
 
 	if jobType == "presubmit" {
-		pullHash := fmt.Sprintf("%x", md5.Sum([]byte(pullNumber)))
-		return fmt.Sprintf("e2e-%v-%v.%v", pullHash[:10], jobHash[:5], suffix), nil
+		return fmt.Sprintf("e2e-pr%s.%s.%s", pullNumber, jobName, suffix), nil
 	}
-	return fmt.Sprintf("e2e-%v-%v.%v", buildIDHash[:10], jobHash[:5], suffix), nil
+	return fmt.Sprintf("e2e-%s.%s", jobName, suffix), nil
 }
 
 // stateStore returns the kops state store to use
