@@ -45,7 +45,6 @@ import (
 	"k8s.io/kops/pkg/dns"
 	"k8s.io/kops/pkg/featureflag"
 	"k8s.io/kops/pkg/model"
-	"k8s.io/kops/pkg/model/alimodel"
 	"k8s.io/kops/pkg/model/awsmodel"
 	"k8s.io/kops/pkg/model/azuremodel"
 	"k8s.io/kops/pkg/model/components"
@@ -53,26 +52,27 @@ import (
 	"k8s.io/kops/pkg/model/components/kubeapiserver"
 	"k8s.io/kops/pkg/model/domodel"
 	"k8s.io/kops/pkg/model/gcemodel"
+	"k8s.io/kops/pkg/model/hetznermodel"
 	"k8s.io/kops/pkg/model/iam"
 	"k8s.io/kops/pkg/model/openstackmodel"
 	"k8s.io/kops/pkg/templates"
 	"k8s.io/kops/pkg/wellknownports"
 	"k8s.io/kops/upup/models"
 	"k8s.io/kops/upup/pkg/fi"
-	"k8s.io/kops/upup/pkg/fi/cloudup/aliup"
 	"k8s.io/kops/upup/pkg/fi/cloudup/awsup"
 	"k8s.io/kops/upup/pkg/fi/cloudup/azure"
 	"k8s.io/kops/upup/pkg/fi/cloudup/bootstrapchannelbuilder"
 	"k8s.io/kops/upup/pkg/fi/cloudup/cloudformation"
 	"k8s.io/kops/upup/pkg/fi/cloudup/do"
 	"k8s.io/kops/upup/pkg/fi/cloudup/gce"
+	"k8s.io/kops/upup/pkg/fi/cloudup/hetzner"
 	"k8s.io/kops/upup/pkg/fi/cloudup/openstack"
 	"k8s.io/kops/upup/pkg/fi/cloudup/terraform"
 	"k8s.io/kops/upup/pkg/fi/cloudup/terraformWriter"
-	"k8s.io/kops/upup/pkg/fi/fitasks"
 	"k8s.io/kops/util/pkg/architectures"
 	"k8s.io/kops/util/pkg/hashing"
 	"k8s.io/kops/util/pkg/mirrors"
+	"k8s.io/kops/util/pkg/reflectutils"
 	"k8s.io/kops/util/pkg/vfs"
 )
 
@@ -80,13 +80,13 @@ const (
 	starline = "*********************************************************************************"
 
 	// OldestSupportedKubernetesVersion is the oldest kubernetes version that is supported in kOps.
-	OldestSupportedKubernetesVersion = "1.18.0"
+	OldestSupportedKubernetesVersion = "1.19.0"
 	// OldestRecommendedKubernetesVersion is the oldest kubernetes version that is not deprecated in kOps.
-	OldestRecommendedKubernetesVersion = "1.20.0"
+	OldestRecommendedKubernetesVersion = "1.21.0"
 )
 
 // TerraformCloudProviders is the list of cloud providers with terraform target support
-var TerraformCloudProviders = []kops.CloudProviderID{kops.CloudProviderAWS, kops.CloudProviderGCE, kops.CloudProviderALI}
+var TerraformCloudProviders = []kops.CloudProviderID{kops.CloudProviderAWS, kops.CloudProviderGCE}
 
 type ApplyClusterCmd struct {
 	Cloud   fi.Cloud
@@ -396,16 +396,18 @@ func (c *ApplyClusterCmd) Run(ctx context.Context) error {
 		InstanceGroups:  c.InstanceGroups,
 	}
 
-	switch kops.CloudProviderID(cluster.Spec.CloudProvider) {
+	switch cluster.Spec.GetCloudProvider() {
 	case kops.CloudProviderGCE:
 		{
 			gceCloud := cloud.(gce.GCECloud)
 			project = gceCloud.Project()
+		}
 
-			if !featureflag.AlphaAllowGCE.Enabled() {
-				return fmt.Errorf("GCE support is currently alpha, and is feature-gated.  export KOPS_FEATURE_FLAGS=AlphaAllowGCE")
+	case kops.CloudProviderHetzner:
+		{
+			if !featureflag.Hetzner.Enabled() {
+				return fmt.Errorf("Hetzner Cloud support is currently alpha, and is feature-gated.  export KOPS_FEATURE_FLAGS=Hetzner")
 			}
-
 		}
 
 	case kops.CloudProviderDO:
@@ -430,24 +432,6 @@ func (c *ApplyClusterCmd) Run(ctx context.Context) error {
 			}
 		}
 
-	case kops.CloudProviderALI:
-		{
-			fmt.Println("")
-			fmt.Println("aliyun support has been deprecated due to lack of maintainers. It may be removed in a future version of kOps.")
-			fmt.Println("")
-
-			if !featureflag.AlphaAllowALI.Enabled() {
-				return fmt.Errorf("aliyun support is currently alpha, and is feature-gated.  export KOPS_FEATURE_FLAGS=AlphaAllowALI")
-			}
-
-			if len(sshPublicKeys) == 0 {
-				return fmt.Errorf("SSH public key must be specified when running with ALICloud (create with `kops create secret --name %s sshpublickey admin -i ~/.ssh/id_rsa.pub`)", cluster.ObjectMeta.Name)
-			}
-
-			if len(sshPublicKeys) != 1 {
-				return fmt.Errorf("exactly one 'admin' SSH public key can be specified when running with ALICloud; please delete a key using `kops delete secret`")
-			}
-		}
 	case kops.CloudProviderAzure:
 		{
 			if !featureflag.Azure.Enabled() {
@@ -473,7 +457,7 @@ func (c *ApplyClusterCmd) Run(ctx context.Context) error {
 			}
 		}
 	default:
-		return fmt.Errorf("unknown CloudProvider %q", cluster.Spec.CloudProvider)
+		return fmt.Errorf("unknown CloudProvider %q", cluster.Spec.GetCloudProvider())
 	}
 
 	modelContext.SSHPublicKeys = sshPublicKeys
@@ -550,7 +534,7 @@ func (c *ApplyClusterCmd) Run(ctx context.Context) error {
 			&model.ConfigBuilder{KopsModelContext: modelContext, Lifecycle: clusterLifecycle},
 		)
 
-		switch kops.CloudProviderID(cluster.Spec.CloudProvider) {
+		switch cluster.Spec.GetCloudProvider() {
 		case kops.CloudProviderAWS:
 			awsModelContext := &awsmodel.AWSModelContext{
 				KopsModelContext: modelContext,
@@ -608,8 +592,19 @@ func (c *ApplyClusterCmd) Run(ctx context.Context) error {
 				&domodel.DropletBuilder{DOModelContext: doModelContext, BootstrapScriptBuilder: bootstrapScriptBuilder, Lifecycle: clusterLifecycle},
 				&domodel.NetworkModelBuilder{DOModelContext: doModelContext, Lifecycle: networkLifecycle},
 			)
+		case kops.CloudProviderHetzner:
+			hetznerModelContext := &hetznermodel.HetznerModelContext{
+				KopsModelContext: modelContext,
+			}
+			l.Builders = append(l.Builders,
+				&hetznermodel.NetworkModelBuilder{HetznerModelContext: hetznerModelContext, Lifecycle: networkLifecycle},
+				&hetznermodel.ExternalAccessModelBuilder{HetznerModelContext: hetznerModelContext, Lifecycle: networkLifecycle},
+				&hetznermodel.LoadBalancerModelBuilder{HetznerModelContext: hetznerModelContext, Lifecycle: networkLifecycle},
+				&hetznermodel.ServerGroupModelBuilder{HetznerModelContext: hetznerModelContext, BootstrapScriptBuilder: bootstrapScriptBuilder, Lifecycle: clusterLifecycle},
+			)
 		case kops.CloudProviderGCE:
 			gceModelContext := &gcemodel.GCEModelContext{
+				ProjectID:        project,
 				KopsModelContext: modelContext,
 			}
 
@@ -627,21 +622,8 @@ func (c *ApplyClusterCmd) Run(ctx context.Context) error {
 				&gcemodel.NetworkModelBuilder{GCEModelContext: gceModelContext, Lifecycle: networkLifecycle},
 				&gcemodel.StorageAclBuilder{GCEModelContext: gceModelContext, Cloud: cloud.(gce.GCECloud), Lifecycle: storageACLLifecycle},
 				&gcemodel.AutoscalingGroupModelBuilder{GCEModelContext: gceModelContext, BootstrapScriptBuilder: bootstrapScriptBuilder, Lifecycle: clusterLifecycle},
+				&gcemodel.ServiceAccountsBuilder{GCEModelContext: gceModelContext, Lifecycle: clusterLifecycle},
 			)
-		case kops.CloudProviderALI:
-			aliModelContext := &alimodel.ALIModelContext{
-				KopsModelContext: modelContext,
-			}
-			l.Builders = append(l.Builders,
-				&alimodel.APILoadBalancerModelBuilder{ALIModelContext: aliModelContext, Lifecycle: clusterLifecycle},
-				&alimodel.NetworkModelBuilder{ALIModelContext: aliModelContext, Lifecycle: clusterLifecycle},
-				&alimodel.RAMModelBuilder{ALIModelContext: aliModelContext, Lifecycle: clusterLifecycle},
-				&alimodel.SSHKeyModelBuilder{ALIModelContext: aliModelContext, Lifecycle: clusterLifecycle},
-				&alimodel.FirewallModelBuilder{ALIModelContext: aliModelContext, Lifecycle: clusterLifecycle},
-				&alimodel.ExternalAccessModelBuilder{ALIModelContext: aliModelContext, Lifecycle: clusterLifecycle},
-				&alimodel.ScalingGroupModelBuilder{ALIModelContext: aliModelContext, BootstrapScriptBuilder: bootstrapScriptBuilder, Lifecycle: clusterLifecycle},
-			)
-
 		case kops.CloudProviderAzure:
 			azureModelContext := &azuremodel.AzureModelContext{
 				KopsModelContext: modelContext,
@@ -666,7 +648,7 @@ func (c *ApplyClusterCmd) Run(ctx context.Context) error {
 			)
 
 		default:
-			return fmt.Errorf("unknown cloudprovider %q", cluster.Spec.CloudProvider)
+			return fmt.Errorf("unknown cloudprovider %q", cluster.Spec.GetCloudProvider())
 		}
 	}
 	c.TaskMap, err = l.BuildTasks(c.LifecycleOverrides)
@@ -679,21 +661,21 @@ func (c *ApplyClusterCmd) Run(ctx context.Context) error {
 
 	switch c.TargetName {
 	case TargetDirect:
-		switch kops.CloudProviderID(cluster.Spec.CloudProvider) {
+		switch cluster.Spec.GetCloudProvider() {
 		case kops.CloudProviderGCE:
 			target = gce.NewGCEAPITarget(cloud.(gce.GCECloud))
 		case kops.CloudProviderAWS:
 			target = awsup.NewAWSAPITarget(cloud.(awsup.AWSCloud))
 		case kops.CloudProviderDO:
 			target = do.NewDOAPITarget(cloud.(do.DOCloud))
+		case kops.CloudProviderHetzner:
+			target = hetzner.NewHetznerAPITarget(cloud.(hetzner.HetznerCloud))
 		case kops.CloudProviderOpenstack:
 			target = openstack.NewOpenstackAPITarget(cloud.(openstack.OpenstackCloud))
-		case kops.CloudProviderALI:
-			target = aliup.NewALIAPITarget(cloud.(aliup.ALICloud))
 		case kops.CloudProviderAzure:
 			target = azure.NewAzureAPITarget(cloud.(azure.AzureCloud))
 		default:
-			return fmt.Errorf("direct configuration not supported with CloudProvider:%q", cluster.Spec.CloudProvider)
+			return fmt.Errorf("direct configuration not supported with CloudProvider:%q", cluster.Spec.GetCloudProvider())
 		}
 
 	case TargetTerraform:
@@ -1111,7 +1093,7 @@ func ChannelForCluster(c *kops.Cluster) (*kops.Channel, error) {
 // This is only needed currently on ContainerOS i.e. GCE, but we don't have a nice way to detect it yet
 func needsMounterAsset(c *kops.Cluster, instanceGroups []*kops.InstanceGroup) bool {
 	// TODO: Do real detection of ContainerOS (but this has to work with image names, and maybe even forked images)
-	switch kops.CloudProviderID(c.Spec.CloudProvider) {
+	switch c.Spec.GetCloudProvider() {
 	case kops.CloudProviderGCE:
 		return true
 	default:
@@ -1284,7 +1266,7 @@ func newNodeUpConfigBuilder(cluster *kops.Cluster, assetBuilder *assets.AssetBui
 }
 
 // BuildConfig returns the NodeUp config and auxiliary config.
-func (n *nodeUpConfigBuilder) BuildConfig(ig *kops.InstanceGroup, apiserverAdditionalIPs []string, caTasks map[string]*fitasks.Keypair) (*nodeup.Config, *nodeup.BootConfig, error) {
+func (n *nodeUpConfigBuilder) BuildConfig(ig *kops.InstanceGroup, apiserverAdditionalIPs []string, keysets map[string]*fi.Keyset) (*nodeup.Config, *nodeup.BootConfig, error) {
 	cluster := n.cluster
 
 	if ig == nil {
@@ -1311,63 +1293,63 @@ func (n *nodeUpConfigBuilder) BuildConfig(ig *kops.InstanceGroup, apiserverAddit
 	}
 
 	if role != kops.InstanceGroupRoleBastion {
-		if err := getTasksCertificate(caTasks, fi.CertificateIDCA, config, true); err != nil {
+		if err := loadCertificates(keysets, fi.CertificateIDCA, config, true); err != nil {
 			return nil, nil, err
 		}
-		if caTasks["etcd-clients-ca-cilium"] != nil {
-			if err := getTasksCertificate(caTasks, "etcd-clients-ca-cilium", config, hasAPIServer || apiModel.UseKopsControllerForNodeBootstrap(n.cluster)); err != nil {
+		if keysets["etcd-clients-ca-cilium"] != nil {
+			if err := loadCertificates(keysets, "etcd-clients-ca-cilium", config, hasAPIServer || apiModel.UseKopsControllerForNodeBootstrap(n.cluster)); err != nil {
 				return nil, nil, err
 			}
 		}
 
 		if isMaster {
-			if err := getTasksCertificate(caTasks, "etcd-clients-ca", config, true); err != nil {
+			if err := loadCertificates(keysets, "etcd-clients-ca", config, true); err != nil {
 				return nil, nil, err
 			}
 			for _, etcdCluster := range cluster.Spec.EtcdClusters {
 				k := etcdCluster.Name
-				if err := getTasksCertificate(caTasks, "etcd-manager-ca-"+k, config, true); err != nil {
+				if err := loadCertificates(keysets, "etcd-manager-ca-"+k, config, true); err != nil {
 					return nil, nil, err
 				}
-				if err := getTasksCertificate(caTasks, "etcd-peers-ca-"+k, config, true); err != nil {
+				if err := loadCertificates(keysets, "etcd-peers-ca-"+k, config, true); err != nil {
 					return nil, nil, err
 				}
 				if k != "events" && k != "main" {
-					if err := getTasksCertificate(caTasks, "etcd-clients-ca-"+k, config, true); err != nil {
+					if err := loadCertificates(keysets, "etcd-clients-ca-"+k, config, true); err != nil {
 						return nil, nil, err
 					}
 				}
 			}
-			config.KeypairIDs["service-account"] = caTasks["service-account"].Keyset().Primary.Id
+			config.KeypairIDs["service-account"] = keysets["service-account"].Primary.Id
 		} else {
-			if caTasks["etcd-client-cilium"] != nil {
-				config.KeypairIDs["etcd-client-cilium"] = caTasks["etcd-client-cilium"].Keyset().Primary.Id
+			if keysets["etcd-client-cilium"] != nil {
+				config.KeypairIDs["etcd-client-cilium"] = keysets["etcd-client-cilium"].Primary.Id
 			}
 		}
 
 		if hasAPIServer {
-			if err := getTasksCertificate(caTasks, "apiserver-aggregator-ca", config, true); err != nil {
+			if err := loadCertificates(keysets, "apiserver-aggregator-ca", config, true); err != nil {
 				return nil, nil, err
 			}
-			if caTasks["etcd-clients-ca"] != nil {
-				if err := getTasksCertificate(caTasks, "etcd-clients-ca", config, true); err != nil {
+			if keysets["etcd-clients-ca"] != nil {
+				if err := loadCertificates(keysets, "etcd-clients-ca", config, true); err != nil {
 					return nil, nil, err
 				}
 			}
 			if cluster.Spec.KubeAPIServer != nil && fi.StringValue(cluster.Spec.KubeAPIServer.ServiceAccountIssuer) != "" {
-				config.KeypairIDs["service-account"] = caTasks["service-account"].Keyset().Primary.Id
+				config.KeypairIDs["service-account"] = keysets["service-account"].Primary.Id
 			}
 
 			config.APIServerConfig.EncryptionConfigSecretHash = n.encryptionConfigSecretHash
-			var err error
-			config.APIServerConfig.ServiceAccountPublicKeys, err = caTasks["service-account"].Keyset().ToPublicKeys()
+			serviceAccountPublicKeys, err := keysets["service-account"].ToPublicKeys()
 			if err != nil {
 				return nil, nil, fmt.Errorf("encoding service-account keys: %w", err)
 			}
+			config.APIServerConfig.ServiceAccountPublicKeys = serviceAccountPublicKeys
 		} else {
 			for _, key := range []string{"kubelet", "kube-proxy", "kube-router"} {
-				if caTasks[key] != nil {
-					config.KeypairIDs[key] = caTasks[key].Keyset().Primary.Id
+				if keysets[key] != nil {
+					config.KeypairIDs[key] = keysets[key].Primary.Id
 				}
 			}
 		}
@@ -1432,34 +1414,71 @@ func (n *nodeUpConfigBuilder) BuildConfig(ig *kops.InstanceGroup, apiserverAddit
 	config.Channels = n.channels
 	config.EtcdManifests = n.etcdManifests[role]
 
-	if cluster.Spec.ContainerRuntime == "containerd" {
-		config.ContainerdConfig = cluster.Spec.Containerd
+	if ig.Spec.Containerd != nil || cluster.Spec.ContainerRuntime == "containerd" {
+		config.ContainerdConfig = n.buildContainerdConfig(ig)
 	}
 
-	if cluster.Spec.Containerd.NvidiaGPU != nil {
-		config.NvidiaGPU = cluster.Spec.Containerd.NvidiaGPU
+	if (cluster.Spec.Containerd != nil && cluster.Spec.Containerd.NvidiaGPU != nil) || (ig.Spec.Containerd != nil && ig.Spec.Containerd.NvidiaGPU != nil) {
+		config.NvidiaGPU = n.buildNvidiaConfig(ig)
 	}
 
 	if ig.Spec.WarmPool != nil || cluster.Spec.WarmPool != nil {
 		config.WarmPoolImages = n.buildWarmPoolImages(ig)
 	}
 
+	if ig.Spec.Packages != nil {
+		config.Packages = ig.Spec.Packages
+	}
+
 	return config, bootConfig, nil
 }
 
-func getTasksCertificate(caTasks map[string]*fitasks.Keypair, name string, config *nodeup.Config, includeKeypairID bool) error {
-	cas, err := fi.ResourceAsString(caTasks[name].Certificates())
-	if err != nil {
-		// CA task may not have run yet; we'll retry
-		return fmt.Errorf("failed to read %s certificates: %w", name, err)
+func loadCertificates(keysets map[string]*fi.Keyset, name string, config *nodeup.Config, includeKeypairID bool) error {
+	keyset := keysets[name]
+	if keyset == nil {
+		return fmt.Errorf("key %q not found", name)
 	}
-	config.CAs[name] = cas
+	certificates, err := keyset.ToCertificateBytes()
+	if err != nil {
+		return fmt.Errorf("failed to read %q certificates: %w", name, err)
+	}
+	config.CAs[name] = string(certificates)
 	if includeKeypairID {
-		config.KeypairIDs[name] = caTasks[name].Keyset().Primary.Id
+		if keyset.Primary == nil {
+			return fmt.Errorf("key %q did not have primary set", name)
+		}
+		config.KeypairIDs[name] = keyset.Primary.Id
 	}
 	return nil
 }
 
+// buildNvidiaConfig builds nvidia configuration for instance group
+func (n *nodeUpConfigBuilder) buildNvidiaConfig(ig *kops.InstanceGroup) *kops.NvidiaGPUConfig {
+	config := &kops.NvidiaGPUConfig{}
+	if n.cluster.Spec.Containerd != nil && n.cluster.Spec.Containerd.NvidiaGPU != nil {
+		config = n.cluster.Spec.Containerd.NvidiaGPU
+	}
+
+	if ig.Spec.Containerd != nil && ig.Spec.Containerd.NvidiaGPU != nil {
+		reflectutils.JSONMergeStruct(&config, ig.Spec.Containerd.NvidiaGPU)
+	}
+
+	if config.DriverPackage == "" {
+		config.DriverPackage = kops.NvidiaDefaultDriverPackage
+	}
+
+	return config
+}
+
+// buildContainerdConfig builds containerd configuration for instance. Instance group configuration will override cluster configuration
+func (n *nodeUpConfigBuilder) buildContainerdConfig(ig *kops.InstanceGroup) *kops.ContainerdConfig {
+	config := n.cluster.Spec.Containerd.DeepCopy()
+	if ig.Spec.Containerd != nil {
+		reflectutils.JSONMergeStruct(&config, ig.Spec.Containerd)
+	}
+	return config
+}
+
 // buildWarmPoolImages returns a list of container images that should be pre-pulled during instance pre-initialization
 func (n *nodeUpConfigBuilder) buildWarmPoolImages(ig *kops.InstanceGroup) []string {
 	if ig == nil || ig.Spec.Role == kops.InstanceGroupRoleMaster {
@@ -1478,10 +1497,10 @@ func (n *nodeUpConfigBuilder) buildWarmPoolImages(ig *kops.InstanceGroup) []stri
 		//"docker.io/cilium/",
 		//"docker.io/cloudnativelabs/kube-router:",
 		//"docker.io/weaveworks/",
-		"k8s.gcr.io/kube-proxy:",
-		"k8s.gcr.io/provider-aws/",
-		"k8s.gcr.io/sig-storage/csi-node-driver-registrar:",
-		"k8s.gcr.io/sig-storage/livenessprobe:",
+		"registry.k8s.io/kube-proxy:",
+		"registry.k8s.io/provider-aws/",
+		"registry.k8s.io/sig-storage/csi-node-driver-registrar:",
+		"registry.k8s.io/sig-storage/livenessprobe:",
 		"quay.io/calico/",
 		"quay.io/cilium/",
 		"quay.io/coreos/flannel:",
