@@ -97,54 +97,3 @@ func Test_FindCNIAssetFromDefaults122(t *testing.T) {
 		t.Errorf("Expected default CNI version hash %q, but got %q instead", desiredCNIVersionHash, cniAssetHash)
 	}
 }
-
-func Test_FindLyftAssetFromEnvironmentVariable(t *testing.T) {
-	desiredLyftVersion := "https://github.com/lyft/cni-ipvlan-vpc-k8s/releases/download/TEST-VERSION/cni-TEST-VERSION.tar.gz"
-	desiredLyftVersionHash := "sha256:0000000000000000000000000000000000000000000000000000000000000000"
-
-	os.Setenv(ENV_VAR_LYFT_VPC_ASSET_URL, desiredLyftVersion)
-	os.Setenv(ENV_VAR_LYFT_VPC_ASSET_HASH, desiredLyftVersionHash)
-	defer func() {
-		os.Unsetenv(ENV_VAR_LYFT_VPC_ASSET_URL)
-		os.Unsetenv(ENV_VAR_LYFT_VPC_ASSET_HASH)
-	}()
-
-	cluster := &api.Cluster{}
-	cluster.Spec.KubernetesVersion = "v1.18.0"
-
-	assetBuilder := assets.NewAssetBuilder(cluster, false)
-	lyftAsset, lyftAssetHash, err := findLyftVPCAssets(cluster, assetBuilder, architectures.ArchitectureAmd64)
-	if err != nil {
-		t.Errorf("Unable to parse Lyft version %s", err)
-	}
-
-	if lyftAsset.String() != desiredLyftVersion {
-		t.Errorf("Expected Lyft version from env var %q, but got %q instead", desiredLyftVersion, lyftAsset)
-	}
-
-	if lyftAssetHash.String() != desiredLyftVersionHash {
-		t.Errorf("Expected Lyft version hash from env var %q, but got %q instead", desiredLyftVersionHash, lyftAssetHash)
-	}
-}
-
-func Test_FindLyftAssetFromDefaults(t *testing.T) {
-	desiredLyftVersion := "https://github.com/lyft/cni-ipvlan-vpc-k8s/releases/download/v0.6.0/cni-ipvlan-vpc-k8s-amd64-v0.6.0.tar.gz"
-	desiredLyftVersionHash := "sha256:871757d381035f64020a523e7a3e139b6177b98eb7a61b547813ff25957fc566"
-
-	cluster := &api.Cluster{}
-	cluster.Spec.KubernetesVersion = "v1.18.0"
-
-	assetBuilder := assets.NewAssetBuilder(cluster, false)
-	lyftAsset, lyftAssetHash, err := findLyftVPCAssets(cluster, assetBuilder, architectures.ArchitectureAmd64)
-	if err != nil {
-		t.Errorf("Unable to parse Lyft version %s", err)
-	}
-
-	if lyftAsset.String() != desiredLyftVersion {
-		t.Errorf("Expected default Lyft version %q, but got %q instead", desiredLyftVersion, lyftAsset)
-	}
-
-	if lyftAssetHash.String() != desiredLyftVersionHash {
-		t.Errorf("Expected default Lyft version hash %q, but got %q instead", desiredLyftVersionHash, lyftAssetHash)
-	}
-}
