@@ -252,8 +252,11 @@ type OpenstackCloud interface {
 	// ListDNSRecordsets will list the DNS recordsets for the given zone id
 	ListDNSRecordsets(zoneID string, opt recordsets.ListOptsBuilder) ([]recordsets.RecordSet, error)
 	GetLB(loadbalancerID string) (*loadbalancers.LoadBalancer, error)
+	GetLBStats(loadbalancerID string) (*loadbalancers.Stats, error)
 	CreateLB(opt loadbalancers.CreateOptsBuilder) (*loadbalancers.LoadBalancer, error)
 	ListLBs(opt loadbalancers.ListOptsBuilder) ([]loadbalancers.LoadBalancer, error)
+	UpdateMemberInPool(poolID string, memberID string, opts v2pools.UpdateMemberOptsBuilder) (*v2pools.Member, error)
+	ListPoolMembers(poolID string, opts v2pools.ListMembersOpts) ([]v2pools.Member, error)
 
 	// DeleteLB will delete loadbalancer
 	DeleteLB(lbID string, opt loadbalancers.DeleteOpts) error
@@ -266,7 +269,8 @@ type OpenstackCloud interface {
 	AssociateToPool(server *servers.Server, poolID string, opts v2pools.CreateMemberOpts) (*v2pools.Member, error)
 	CreatePool(opts v2pools.CreateOpts) (*v2pools.Pool, error)
 	CreatePoolMonitor(opts monitors.CreateOpts) (*monitors.Monitor, error)
-	GetPool(poolID string, memberID string) (*v2pools.Member, error)
+	GetPool(poolID string) (*v2pools.Pool, error)
+	GetPoolMember(poolID string, memberID string) (*v2pools.Member, error)
 	ListPools(v2pools.ListOpts) ([]v2pools.Pool, error)
 
 	// ListMonitors will list HealthMonitors matching the provided options
@@ -314,7 +318,7 @@ type openstackCloud struct {
 
 var _ fi.Cloud = &openstackCloud{}
 
-var openstackCloudInstances map[string]OpenstackCloud = make(map[string]OpenstackCloud)
+var openstackCloudInstances = make(map[string]OpenstackCloud)
 
 func NewOpenstackCloud(tags map[string]string, spec *kops.ClusterSpec, uagent string) (OpenstackCloud, error) {
 	config := vfs.OpenstackConfig{}
@@ -343,9 +347,9 @@ func NewOpenstackCloud(tags map[string]string, spec *kops.ClusterSpec, uagent st
 	provider.UserAgent = ua
 	klog.V(4).Infof("Using user-agent %s", ua.Join())
 
-	if spec != nil && spec.CloudConfig != nil && spec.CloudConfig.Openstack != nil && spec.CloudConfig.Openstack.InsecureSkipVerify != nil {
+	if spec != nil && spec.CloudProvider.Openstack != nil && spec.CloudProvider.Openstack.InsecureSkipVerify != nil {
 		tlsconfig := &tls.Config{}
-		tlsconfig.InsecureSkipVerify = fi.BoolValue(spec.CloudConfig.Openstack.InsecureSkipVerify)
+		tlsconfig.InsecureSkipVerify = fi.BoolValue(spec.CloudProvider.Openstack.InsecureSkipVerify)
 		transport := &http.Transport{TLSClientConfig: tlsconfig}
 		provider.HTTPClient = http.Client{
 			Transport: transport,
@@ -421,42 +425,41 @@ func NewOpenstackCloud(tags map[string]string, spec *kops.ClusterSpec, uagent st
 	octavia := false
 	floatingEnabled := false
 	if spec != nil &&
-		spec.CloudConfig != nil &&
-		spec.CloudConfig.Openstack != nil &&
-		spec.CloudConfig.Openstack.Router != nil {
+		spec.CloudProvider.Openstack != nil &&
+		spec.CloudProvider.Openstack.Router != nil {
 
 		floatingEnabled = true
-		c.extNetworkName = spec.CloudConfig.Openstack.Router.ExternalNetwork
+		c.extNetworkName = spec.CloudProvider.Openstack.Router.ExternalNetwork
 
-		if spec.CloudConfig.Openstack.Router.ExternalSubnet != nil {
-			c.extSubnetName = spec.CloudConfig.Openstack.Router.ExternalSubnet
+		if spec.CloudProvider.Openstack.Router.ExternalSubnet != nil {
+			c.extSubnetName = spec.CloudProvider.Openstack.Router.ExternalSubnet
 		}
-		if spec.CloudConfig.Openstack.Loadbalancer != nil &&
-			spec.CloudConfig.Openstack.Loadbalancer.FloatingNetworkID == nil &&
-			spec.CloudConfig.Openstack.Loadbalancer.FloatingNetwork != nil {
+		if spec.CloudProvider.Openstack.Loadbalancer != nil &&
+			spec.CloudProvider.Openstack.Loadbalancer.FloatingNetworkID == nil &&
+			spec.CloudProvider.Openstack.Loadbalancer.FloatingNetwork != nil {
 			// This field is derived
 			lbNet, err := c.ListNetworks(networks.ListOpts{
-				Name: fi.StringValue(spec.CloudConfig.Openstack.Loadbalancer.FloatingNetwork),
+				Name: fi.StringValue(spec.CloudProvider.Openstack.Loadbalancer.FloatingNetwork),
 			})
 			if err != nil || len(lbNet) != 1 {
 				return c, fmt.Errorf("could not establish floating network id")
 			}
-			spec.CloudConfig.Openstack.Loadbalancer.FloatingNetworkID = fi.String(lbNet[0].ID)
+			spec.CloudProvider.Openstack.Loadbalancer.FloatingNetworkID = fi.String(lbNet[0].ID)
 		}
-		if spec.CloudConfig.Openstack.Loadbalancer != nil {
-			if spec.CloudConfig.Openstack.Loadbalancer.UseOctavia != nil {
-				octavia = fi.BoolValue(spec.CloudConfig.Openstack.Loadbalancer.UseOctavia)
+		if spec.CloudProvider.Openstack.Loadbalancer != nil {
+			if spec.CloudProvider.Openstack.Loadbalancer.UseOctavia != nil {
+				octavia = fi.BoolValue(spec.CloudProvider.Openstack.Loadbalancer.UseOctavia)
 			}
-			if spec.CloudConfig.Openstack.Loadbalancer.FloatingSubnet != nil {
-				c.floatingSubnet = spec.CloudConfig.Openstack.Loadbalancer.FloatingSubnet
+			if spec.CloudProvider.Openstack.Loadbalancer.FloatingSubnet != nil {
+				c.floatingSubnet = spec.CloudProvider.Openstack.Loadbalancer.FloatingSubnet
 			}
 		}
 	}
 	c.floatingEnabled = floatingEnabled
 	c.useOctavia = octavia
 	var lbClient *gophercloud.ServiceClient
-	if spec != nil && spec.CloudConfig != nil && spec.CloudConfig.Openstack != nil {
-		if spec.CloudConfig.Openstack.Loadbalancer != nil && octavia {
+	if spec != nil && spec.CloudProvider.Openstack != nil {
+		if spec.CloudProvider.Openstack.Loadbalancer != nil && octavia {
 			klog.V(2).Infof("Openstack using Octavia lbaasv2 api")
 			lbClient, err = os.NewLoadBalancerV2(provider, gophercloud.EndpointOpts{
 				Region: region,
@@ -464,7 +467,7 @@ func NewOpenstackCloud(tags map[string]string, spec *kops.ClusterSpec, uagent st
 			if err != nil {
 				return nil, fmt.Errorf("error building lb client: %v", err)
 			}
-		} else if spec.CloudConfig.Openstack.Loadbalancer != nil {
+		} else if spec.CloudProvider.Openstack.Loadbalancer != nil {
 			klog.V(2).Infof("Openstack using deprecated lbaasv2 api")
 			lbClient, err = os.NewNetworkV2(provider, gophercloud.EndpointOpts{
 				Region: region,
@@ -681,7 +684,7 @@ func (c *openstackCloud) GetApiIngressStatus(cluster *kops.Cluster) ([]fi.ApiIng
 }
 
 func getApiIngressStatus(c OpenstackCloud, cluster *kops.Cluster) ([]fi.ApiIngressStatus, error) {
-	if cluster.Spec.CloudConfig.Openstack.Loadbalancer != nil {
+	if cluster.Spec.CloudProvider.Openstack.Loadbalancer != nil {
 		return getLoadBalancerIngressStatus(c, cluster)
 	} else {
 		return getIPIngressStatus(c, cluster)
