@@ -195,59 +195,13 @@ func build(c *kopsapi.Cluster) (*kopsapi.Cluster, error) {
 	return full, nil
 }
 
-func TestPopulateCluster_Kubenet(t *testing.T) {
-	_, c := buildMinimalCluster()
-	c.Spec.ContainerRuntime = "docker"
-
-	full, err := build(c)
-	if err != nil {
-		t.Fatalf("error during build: %v", err)
-	}
-
-	if full.Spec.Kubelet.NetworkPluginName != "kubenet" {
-		t.Fatalf("Unexpected NetworkPluginName: %v", full.Spec.Kubelet.NetworkPluginName)
-	}
-
-	if fi.BoolValue(full.Spec.KubeControllerManager.ConfigureCloudRoutes) != true {
-		t.Fatalf("Unexpected ConfigureCloudRoutes: %v", full.Spec.KubeControllerManager.ConfigureCloudRoutes)
-	}
-}
-
-func TestPopulateCluster_CNI(t *testing.T) {
-	_, c := buildMinimalCluster()
-
-	c.Spec.Kubelet = &kopsapi.KubeletConfigSpec{
-		ConfigureCBR0:     fi.Bool(false),
-		NetworkPluginName: "cni",
-		NonMasqueradeCIDR: c.Spec.NonMasqueradeCIDR,
-		CloudProvider:     c.Spec.CloudProvider,
-	}
-
-	full, err := build(c)
-	if err != nil {
-		t.Fatalf("error during build: %v", err)
-	}
-
-	if full.Spec.Kubelet.NetworkPluginName != "cni" {
-		t.Fatalf("Unexpected NetworkPluginName: %v", full.Spec.Kubelet.NetworkPluginName)
-	}
-
-	if fi.BoolValue(full.Spec.Kubelet.ConfigureCBR0) != false {
-		t.Fatalf("Unexpected ConfigureCBR0: %v", full.Spec.Kubelet.ConfigureCBR0)
-	}
-
-	if fi.BoolValue(full.Spec.KubeControllerManager.ConfigureCloudRoutes) != true {
-		t.Fatalf("Unexpected ConfigureCloudRoutes: %v", full.Spec.KubeControllerManager.ConfigureCloudRoutes)
-	}
-}
-
 func TestPopulateCluster_Custom_CIDR(t *testing.T) {
 	cloud, c := buildMinimalCluster()
 	c.Spec.NetworkCIDR = "172.20.2.0/24"
 	c.Spec.Subnets = []kopsapi.ClusterSubnetSpec{
-		{Name: "subnet-us-mock-1a", Zone: "us-mock-1a", CIDR: "172.20.2.0/27"},
-		{Name: "subnet-us-mock-1b", Zone: "us-mock-1b", CIDR: "172.20.2.32/27"},
-		{Name: "subnet-us-mock-1c", Zone: "us-mock-1c", CIDR: "172.20.2.64/27"},
+		{Name: "subnet-us-mock-1a", Zone: "us-mock-1a", CIDR: "172.20.2.0/27", Type: kopsapi.SubnetTypePublic},
+		{Name: "subnet-us-mock-1b", Zone: "us-mock-1b", CIDR: "172.20.2.32/27", Type: kopsapi.SubnetTypePublic},
+		{Name: "subnet-us-mock-1c", Zone: "us-mock-1c", CIDR: "172.20.2.64/27", Type: kopsapi.SubnetTypePublic},
 	}
 
 	err := PerformAssignments(c, cloud)
@@ -333,7 +287,7 @@ func TestPopulateCluster_NonMasqueradeCIDR_Required(t *testing.T) {
 
 func TestPopulateCluster_CloudProvider_Required(t *testing.T) {
 	cloud, c := buildMinimalCluster()
-	c.Spec.CloudProvider = ""
+	c.Spec.CloudProvider = kopsapi.CloudProviderSpec{}
 
 	expectErrorFromPopulateCluster(t, c, cloud, "cloudProvider")
 }
@@ -437,7 +391,7 @@ func TestPopulateCluster_DockerVersion(t *testing.T) {
 		},
 		{
 			KubernetesVersion: "1.21.0",
-			DockerVersion:     "20.10.9",
+			DockerVersion:     "20.10.17",
 		},
 	}
 
