@@ -30,6 +30,9 @@ func buildMinimalNodeInstanceGroup(subnets ...string) *kopsapi.InstanceGroup {
 	g := &kopsapi.InstanceGroup{}
 	g.ObjectMeta.Name = "nodes"
 	g.Spec.Role = kopsapi.InstanceGroupRoleNode
+	g.Spec.MinSize = fi.Int32(1)
+	g.Spec.MaxSize = fi.Int32(1)
+	g.Spec.Image = "my-image"
 	g.Spec.Subnets = subnets
 
 	return g
@@ -39,6 +42,9 @@ func buildMinimalMasterInstanceGroup(subnet string) *kopsapi.InstanceGroup {
 	g := &kopsapi.InstanceGroup{}
 	g.ObjectMeta.Name = "master-" + subnet
 	g.Spec.Role = kopsapi.InstanceGroupRoleMaster
+	g.Spec.MinSize = fi.Int32(1)
+	g.Spec.MaxSize = fi.Int32(1)
+	g.Spec.Image = "my-image"
 	g.Spec.Subnets = []string{subnet}
 
 	return g
@@ -64,10 +70,19 @@ func TestPopulateInstanceGroup_Role_Required(t *testing.T) {
 	expectErrorFromPopulateInstanceGroup(t, cluster, g, channel, "spec.role")
 }
 
+func TestPopulateInstanceGroup_Image_Required(t *testing.T) {
+	_, cluster := buildMinimalCluster()
+	g := buildMinimalNodeInstanceGroup()
+	g.Spec.Image = ""
+
+	channel := &kopsapi.Channel{}
+
+	expectErrorFromPopulateInstanceGroup(t, cluster, g, channel, "unable to determine default image for InstanceGroup nodes")
+}
+
 func TestPopulateInstanceGroup_AddTaintsCollision(t *testing.T) {
 	_, cluster := buildMinimalCluster()
 	input := buildMinimalNodeInstanceGroup()
-	input.Spec.Image = "my-image"
 	input.Spec.Taints = []string{"nvidia.com/gpu:NoSchedule"}
 	input.Spec.MachineType = "g4dn.xlarge"
 	cluster.Spec.Containerd.NvidiaGPU = &kopsapi.NvidiaGPUConfig{Enabled: fi.Bool(true)}
@@ -90,7 +105,6 @@ func TestPopulateInstanceGroup_AddTaintsCollision(t *testing.T) {
 func TestPopulateInstanceGroup_AddTaints(t *testing.T) {
 	_, cluster := buildMinimalCluster()
 	input := buildMinimalNodeInstanceGroup()
-	input.Spec.Image = "my-image"
 	input.Spec.MachineType = "g4dn.xlarge"
 	cluster.Spec.Containerd.NvidiaGPU = &kopsapi.NvidiaGPUConfig{Enabled: fi.Bool(true)}
 
