@@ -40,16 +40,21 @@ import (
 	"text/template"
 
 	"github.com/Masterminds/sprig/v3"
+	"github.com/aws/aws-sdk-go/service/ec2"
 	corev1 "k8s.io/api/core/v1"
+	"k8s.io/apimachinery/pkg/api/resource"
 	"k8s.io/apimachinery/pkg/util/sets"
 	"k8s.io/klog/v2"
+	"k8s.io/kops/pkg/flagbuilder"
+	"sigs.k8s.io/yaml"
+
 	kopscontrollerconfig "k8s.io/kops/cmd/kops-controller/pkg/config"
 	"k8s.io/kops/pkg/apis/kops"
 	apiModel "k8s.io/kops/pkg/apis/kops/model"
 	"k8s.io/kops/pkg/apis/kops/util"
+	"k8s.io/kops/pkg/apis/nodeup"
 	"k8s.io/kops/pkg/dns"
 	"k8s.io/kops/pkg/featureflag"
-	"k8s.io/kops/pkg/flagbuilder"
 	"k8s.io/kops/pkg/kubemanifest"
 	"k8s.io/kops/pkg/model"
 	"k8s.io/kops/pkg/model/components/kopscontroller"
@@ -61,7 +66,6 @@ import (
 	"k8s.io/kops/upup/pkg/fi/cloudup/gce"
 	gcetpm "k8s.io/kops/upup/pkg/fi/cloudup/gce/tpm"
 	"k8s.io/kops/util/pkg/env"
-	"sigs.k8s.io/yaml"
 )
 
 // TemplateFunctions provides a collection of methods used throughout the templates
@@ -98,6 +102,7 @@ func (tf *TemplateFunctions) AddTo(dest template.FuncMap, secretStore fi.SecretS
 	dest["contains"] = sprigTxtFuncMap["contains"]
 	dest["trimPrefix"] = sprigTxtFuncMap["trimPrefix"]
 	dest["semverCompare"] = sprigTxtFuncMap["semverCompare"]
+	dest["ternary"] = sprigTxtFuncMap["ternary"]
 
 	dest["ClusterName"] = tf.ClusterName
 	dest["WithDefaultBool"] = func(v *bool, defaultValue bool) bool {
@@ -107,6 +112,7 @@ func (tf *TemplateFunctions) AddTo(dest template.FuncMap, secretStore fi.SecretS
 		return defaultValue
 	}
 
+	dest["GetCloudProvider"] = cluster.Spec.GetCloudProvider
 	dest["GetInstanceGroup"] = tf.GetInstanceGroup
 	dest["GetNodeInstanceGroups"] = tf.GetNodeInstanceGroups
 	dest["HasHighlyAvailableControlPlane"] = tf.HasHighlyAvailableControlPlane
@@ -164,6 +170,10 @@ func (tf *TemplateFunctions) AddTo(dest template.FuncMap, secretStore fi.SecretS
 		return os.Getenv("DIGITALOCEAN_ACCESS_TOKEN")
 	}
 
+	dest["HCLOUD_TOKEN"] = func() string {
+		return os.Getenv("HCLOUD_TOKEN")
+	}
+
 	if featureflag.Spotinst.Enabled() {
 		if creds, err := spotinst.LoadCredentials(); err == nil {
 			dest["SpotinstToken"] = func() string { return creds.Token }
@@ -177,7 +187,24 @@ func (tf *TemplateFunctions) AddTo(dest template.FuncMap, secretStore fi.SecretS
 		c := cluster.Spec.Networking.AmazonVPC
 		dest["AmazonVpcEnvVars"] = func() map[string]string {
 			envVars := map[string]string{
-				"AWS_VPC_K8S_CNI_CONFIGURE_RPFILTER": "false",
+				// Use defaults from the official AWS VPC CNI Helm chart:
+				// https://github.com/aws/amazon-vpc-cni-k8s/blob/master/charts/aws-vpc-cni/values.yaml
+				"AWS_VPC_CNI_NODE_PORT_SUPPORT":         "true",
+				"AWS_VPC_ENI_MTU":                       "9001",
+				"AWS_VPC_K8S_CNI_CONFIGURE_RPFILTER":    "false",
+				"AWS_VPC_K8S_CNI_CUSTOM_NETWORK_CFG":    "false",
+				"AWS_VPC_K8S_CNI_EXTERNALSNAT":          "false",
+				"AWS_VPC_K8S_CNI_LOG_FILE":              "/host/var/log/aws-routed-eni/ipamd.log",
+				"AWS_VPC_K8S_CNI_LOGLEVEL":              "DEBUG",
+				"AWS_VPC_K8S_CNI_RANDOMIZESNAT":         "prng",
+				"AWS_VPC_K8S_CNI_VETHPREFIX":            "eni",
+				"AWS_VPC_K8S_PLUGIN_LOG_FILE":           "/var/log/aws-routed-eni/plugin.log",
+				"AWS_VPC_K8S_PLUGIN_LOG_LEVEL":          "DEBUG",
+				"DISABLE_INTROSPECTION":                 "false",
+				"DISABLE_METRICS":                       "false",
+				"ENABLE_POD_ENI":                        "false",
+				"WARM_ENI_TARGET":                       "1",
+				"DISABLE_NETWORK_RESOURCE_PROVISIONING": "false",
 			}
 			for _, e := range c.Env {
 				envVars[e.Name] = e.Value
@@ -188,6 +215,12 @@ func (tf *TemplateFunctions) AddTo(dest template.FuncMap, secretStore fi.SecretS
 				envVars["ENABLE_PREFIX_DELEGATION"] = "true"
 				envVars["WARM_PREFIX_TARGET"] = "1"
 			}
+			envVars["ADDITIONAL_ENI_TAGS"] = fmt.Sprintf(
+				"{\\\"KubernetesCluster\\\":\\\"%s\\\",\\\"kubernetes.io/cluster/%s\\\":\\\"owned\\\"}",
+				tf.ClusterName(),
+				tf.ClusterName(),
+			)
+
 			return envVars
 		}
 	}
@@ -201,7 +234,7 @@ func (tf *TemplateFunctions) AddTo(dest template.FuncMap, secretStore fi.SecretS
 			if c.IPIPMode != "" {
 				return c.IPIPMode
 			}
-			if kops.CloudProviderID(cluster.Spec.CloudProvider) == kops.CloudProviderOpenstack {
+			if cluster.Spec.GetCloudProvider() == kops.CloudProviderOpenstack {
 				return "Always"
 			}
 			return "CrossSubnet"
@@ -283,10 +316,27 @@ func (tf *TemplateFunctions) AddTo(dest template.FuncMap, secretStore fi.SecretS
 		dest["EnableSQSTerminationDraining"] = func() bool { return *cluster.Spec.NodeTerminationHandler.EnableSQSTerminationDraining }
 	}
 
+	dest["ArchitectureOfAMI"] = tf.architectureOfAMI
+
 	dest["ParseTaint"] = util.ParseTaint
 
+	dest["UsesInstanceIDForNodeName"] = func() bool {
+		return nodeup.UsesInstanceIDForNodeName(tf.Cluster)
+	}
+
+	dest["KarpenterInstanceTypes"] = func(ig kops.InstanceGroupSpec) ([]string, error) {
+		return karpenterInstanceTypes(tf.cloud.(awsup.AWSCloud), ig)
+	}
+
 	dest["PodIdentityWebhookConfigMapData"] = tf.podIdentityWebhookConfigMapData
 
+	dest["HasSnapshotController"] = func() bool {
+		sc := cluster.Spec.SnapshotController
+		return sc != nil && fi.BoolValue(sc.Enabled)
+	}
+
+	dest["IsKubernetesLT"] = tf.IsKubernetesLT
+
 	return nil
 }
 
@@ -328,8 +378,8 @@ func (tf *TemplateFunctions) GetInstanceGroup(name string) (*kops.InstanceGroup,
 // If the cluster has a highly available control plane, this function will return 2, if it has 1 control plane node, it will return 1
 // deployOnWorkersIfExternalPermissons should be true if a controller runs on worker nodes when external IAM permissions is enabled for the cluster.
 // In this case it is assumed that it can run 2 replicas.
-func (tf *TemplateFunctions) ControlPlaneControllerReplicas(deployOnWorkersIfExternalPermissons bool) int {
-	if deployOnWorkersIfExternalPermissons && fi.BoolValue(tf.Cluster.Spec.IAM.UseServiceAccountExternalPermissions) {
+func (tf *TemplateFunctions) ControlPlaneControllerReplicas(deployOnWorkersIfExternalPermissions bool) int {
+	if deployOnWorkersIfExternalPermissions && tf.Cluster.Spec.IAM != nil && fi.BoolValue(tf.Cluster.Spec.IAM.UseServiceAccountExternalPermissions) {
 		return 2
 	}
 	if tf.HasHighlyAvailableControlPlane() {
@@ -341,6 +391,8 @@ func (tf *TemplateFunctions) ControlPlaneControllerReplicas(deployOnWorkersIfExt
 func (tf *TemplateFunctions) APIServerNodeRole() string {
 	if featureflag.APIServerNodes.Enabled() {
 		return "node-role.kubernetes.io/api-server"
+	} else if tf.Cluster.IsKubernetesGTE("1.24") {
+		return "node-role.kubernetes.io/control-plane"
 	}
 	return "node-role.kubernetes.io/master"
 }
@@ -379,8 +431,8 @@ func (tf *TemplateFunctions) CloudControllerConfigArgv() ([]string, error) {
 
 	// take the cloud provider value from clusterSpec if unset
 	if cluster.Spec.ExternalCloudControllerManager.CloudProvider == "" {
-		if cluster.Spec.CloudProvider != "" {
-			argv = append(argv, fmt.Sprintf("--cloud-provider=%s", cluster.Spec.CloudProvider))
+		if cluster.Spec.GetCloudProvider() != "" {
+			argv = append(argv, fmt.Sprintf("--cloud-provider=%s", cluster.Spec.GetCloudProvider()))
 		} else {
 			return nil, fmt.Errorf("Cloud Provider is not set")
 		}
@@ -402,8 +454,6 @@ func (tf *TemplateFunctions) DNSControllerArgv() ([]string, error) {
 
 	var argv []string
 
-	argv = append(argv, "/dns-controller")
-
 	// @check if the dns controller has custom configuration
 	if cluster.Spec.ExternalDNS == nil {
 		argv = append(argv, []string{"--watch-ingress=false"}...)
@@ -473,7 +523,7 @@ func (tf *TemplateFunctions) DNSControllerArgv() ([]string, error) {
 			argv = append(argv, fmt.Sprintf("--gossip-seed-secondary=127.0.0.1:%d", wellknownports.ProtokubeGossipMemberlist))
 		}
 	} else {
-		switch kops.CloudProviderID(cluster.Spec.CloudProvider) {
+		switch cluster.Spec.GetCloudProvider() {
 		case kops.CloudProviderAWS:
 			if strings.HasPrefix(os.Getenv("AWS_REGION"), "cn-") {
 				argv = append(argv, "--dns=gossip")
@@ -486,7 +536,7 @@ func (tf *TemplateFunctions) DNSControllerArgv() ([]string, error) {
 			argv = append(argv, "--dns=digitalocean")
 
 		default:
-			return nil, fmt.Errorf("unhandled cloudprovider %q", cluster.Spec.CloudProvider)
+			return nil, fmt.Errorf("unhandled cloudprovider %q", cluster.Spec.GetCloudProvider())
 		}
 	}
 
@@ -520,7 +570,7 @@ func (tf *TemplateFunctions) KopsControllerConfig() (string, error) {
 	cluster := tf.Cluster
 
 	config := &kopscontrollerconfig.Options{
-		Cloud:      cluster.Spec.CloudProvider,
+		Cloud:      string(cluster.Spec.GetCloudProvider()),
 		ConfigBase: cluster.Spec.ConfigBase,
 	}
 
@@ -552,7 +602,7 @@ func (tf *TemplateFunctions) KopsControllerConfig() (string, error) {
 			CertNames:             certNames,
 		}
 
-		switch kops.CloudProviderID(cluster.Spec.CloudProvider) {
+		switch cluster.Spec.GetCloudProvider() {
 		case kops.CloudProviderAWS:
 			nodesRoles := sets.String{}
 			for _, ig := range tf.InstanceGroups {
@@ -598,7 +648,7 @@ func (tf *TemplateFunctions) KopsControllerConfig() (string, error) {
 				MaxTimeSkew: 300,
 			}
 		default:
-			return "", fmt.Errorf("unsupported cloud provider %s", cluster.Spec.CloudProvider)
+			return "", fmt.Errorf("unsupported cloud provider %s", cluster.Spec.GetCloudProvider())
 		}
 	}
 
@@ -625,8 +675,6 @@ func (tf *TemplateFunctions) KopsControllerConfig() (string, error) {
 func (tf *TemplateFunctions) KopsControllerArgv() ([]string, error) {
 	var argv []string
 
-	argv = append(argv, "/kops-controller")
-
 	// Verbose, but not excessive logging
 	argv = append(argv, "--v=2")
 
@@ -641,9 +689,9 @@ func (tf *TemplateFunctions) ExternalDNSArgv() ([]string, error) {
 
 	var argv []string
 
-	cloudProvider := cluster.Spec.CloudProvider
+	cloudProvider := cluster.Spec.GetCloudProvider()
 
-	switch kops.CloudProviderID(cloudProvider) {
+	switch cloudProvider {
 	case kops.CloudProviderAWS:
 		argv = append(argv, "--provider=aws")
 	case kops.CloudProviderGCE:
@@ -651,7 +699,7 @@ func (tf *TemplateFunctions) ExternalDNSArgv() ([]string, error) {
 		argv = append(argv, "--provider=google")
 		argv = append(argv, "--google-project="+project)
 	default:
-		return nil, fmt.Errorf("unhandled cloudprovider %q", cluster.Spec.CloudProvider)
+		return nil, fmt.Errorf("unhandled cloudprovider %q", cluster.Spec.GetCloudProvider())
 	}
 
 	argv = append(argv, "--events")
@@ -719,6 +767,8 @@ func (tf *TemplateFunctions) OpenStackCCMTag() string {
 		} else if parsed.Minor == 23 {
 			// The bugfix release, see https://github.com/kubernetes/cloud-provider-openstack/releases
 			tag = "v1.23.1"
+		} else if parsed.Minor == 24 {
+			tag = "v1.24.1"
 		} else {
 			// otherwise we use always .0 ccm image, if needed that can be overrided using clusterspec
 			tag = fmt.Sprintf("v%d.%d.0", parsed.Major, parsed.Minor)
@@ -735,8 +785,12 @@ func (tf *TemplateFunctions) OpenStackCSITag() string {
 	if err != nil {
 		tag = "latest"
 	} else {
-		// otherwise we use always .0 csi image, if needed that can be overrided using cloud config spec
-		tag = fmt.Sprintf("v%d.%d.0", parsed.Major, parsed.Minor)
+		if parsed.Minor == 24 {
+			tag = "v1.24.1"
+		} else {
+			// otherwise we use always .0 csi image, if needed that can be overrided using cloud config spec
+			tag = fmt.Sprintf("v%d.%d.0", parsed.Major, parsed.Minor)
+		}
 	}
 	return tag
 }
@@ -752,6 +806,15 @@ func (tf *TemplateFunctions) GetNodeInstanceGroups() map[string]kops.InstanceGro
 	return nodegroups
 }
 
+func (tf *TemplateFunctions) architectureOfAMI(amiID string) string {
+	image, _ := tf.cloud.(awsup.AWSCloud).ResolveImage(amiID)
+	switch *image.Architecture {
+	case "x86_64":
+		return "amd64"
+	}
+	return "arm64"
+}
+
 type podIdentityWebhookMapping struct {
 	RoleARN         string
 	Audience        string
@@ -776,3 +839,111 @@ func (tf *TemplateFunctions) podIdentityWebhookConfigMapData() (string, error) {
 	jsonBytes, err := json.Marshal(mappings)
 	return fmt.Sprintf("%q", jsonBytes), err
 }
+
+func karpenterInstanceTypes(cloud awsup.AWSCloud, ig kops.InstanceGroupSpec) ([]string, error) {
+	var mixedInstancesPolicy *kops.MixedInstancesPolicySpec
+
+	if ig.MachineType == "" && ig.MixedInstancesPolicy == nil {
+		// Karpenter thinks all clusters run VPC CNI and schedules thinking Node Capacity is constrainted by number of ENIs.
+
+		// cpuMin is the reasonable lower limit for a Kubernetes Node
+		// Generally, it also avoids instances Karpenter thinks it can only schedule 4 Pods on.
+		cpuMin := resource.MustParse("2")
+		memoryMin := resource.MustParse(("2G"))
+
+		mixedInstancesPolicy = &kops.MixedInstancesPolicySpec{
+			InstanceRequirements: &kops.InstanceRequirementsSpec{
+				CPU: &kops.MinMaxSpec{
+					Min: &cpuMin,
+				},
+				Memory: &kops.MinMaxSpec{
+					Min: &memoryMin,
+				},
+			},
+		}
+	}
+	if ig.MixedInstancesPolicy != nil {
+		mixedInstancesPolicy = ig.MixedInstancesPolicy
+	}
+
+	if mixedInstancesPolicy != nil {
+		if len(mixedInstancesPolicy.Instances) > 0 {
+			return mixedInstancesPolicy.Instances, nil
+		}
+		if mixedInstancesPolicy.InstanceRequirements != nil {
+			instanceRequirements := mixedInstancesPolicy.InstanceRequirements
+			ami, err := cloud.ResolveImage(ig.Image)
+			if err != nil {
+				return nil, err
+			}
+			arch := ami.Architecture
+			hv := ami.VirtualizationType
+
+			ir := &ec2.InstanceRequirementsRequest{
+				VCpuCount:            &ec2.VCpuCountRangeRequest{},
+				MemoryMiB:            &ec2.MemoryMiBRequest{},
+				BurstablePerformance: fi.String("included"),
+			}
+			cpu := instanceRequirements.CPU
+			if cpu != nil {
+				if cpu.Max != nil {
+					cpuMax, _ := instanceRequirements.CPU.Max.AsInt64()
+					ir.VCpuCount.Max = &cpuMax
+				}
+				cpu := instanceRequirements.CPU
+				if cpu != nil {
+					if cpu.Max != nil {
+						cpuMax, _ := instanceRequirements.CPU.Max.AsInt64()
+						ir.VCpuCount.Max = &cpuMax
+					}
+					if cpu.Min != nil {
+						cpuMin, _ := instanceRequirements.CPU.Min.AsInt64()
+						ir.VCpuCount.Min = &cpuMin
+					}
+				} else {
+					ir.VCpuCount.Min = fi.Int64(0)
+				}
+
+				memory := instanceRequirements.Memory
+				if memory != nil {
+					if memory.Max != nil {
+						memoryMax := instanceRequirements.Memory.Max.ScaledValue(resource.Mega)
+						ir.MemoryMiB.Max = &memoryMax
+					}
+					if memory.Min != nil {
+						memoryMin := instanceRequirements.Memory.Min.ScaledValue(resource.Mega)
+						ir.MemoryMiB.Min = &memoryMin
+					}
+				} else {
+					ir.MemoryMiB.Min = fi.Int64(0)
+				}
+
+				ir.AcceleratorCount = &ec2.AcceleratorCountRequest{
+					Min: fi.Int64(0),
+					Max: fi.Int64(0),
+				}
+
+				response, err := cloud.EC2().GetInstanceTypesFromInstanceRequirements(
+					&ec2.GetInstanceTypesFromInstanceRequirementsInput{
+						ArchitectureTypes:    []*string{arch},
+						VirtualizationTypes:  []*string{hv},
+						InstanceRequirements: ir,
+					},
+				)
+				if err != nil {
+					return nil, err
+				}
+				types := []string{}
+				for _, it := range response.InstanceTypes {
+					types = append(types, *it.InstanceType)
+				}
+				if len(types) == 0 {
+					return nil, fmt.Errorf("no instances matched requirements")
+				}
+				return types, nil
+			}
+		}
+	}
+
+	return []string{ig.MachineType}, nil
+}
